//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCDAAutoCal.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function  
//				dscDAAutoCal(). This sample code describes the usage
//				for Hercules II EBX SBC. AutoCalibration will use the onboard
//              hardware to calibrate the D/A circuitry on the board,
//              then verify that the process results fall within tolerances.
//              The calibration values will be stored in the EEPROM,
//              along with one set of values to be loaded at boot time.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		07/27/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>

#include "dscud.h"

#include <sys/time.h>

#endif

// macro declarations
#define OFFSET_LSB_TOLERANCE 2
#define GAIN_LSB_TOLERANCE 2

// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCB dsccb;  // structure containing board settings
DSCDACALPARAMS dscdacalparams;// structure containing auto-calibration settings
ERRPARAMS errorParams;        // structure returns error messages
int intBuff;       // integer variable
long longBuff;	   // long variable
float floatBuff;   // float variable

int i;	// miscellaneous counter

char input_buffer[20]; // Buffer for parsing user character input.

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AutoCal Settings Initialization
//	    IV. Auto-Calibration
//		 V. Calibration Verification
//	    VI. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( dscInit ( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the HERCULES board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type
	//	   2. set the base address
	//	   3. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf ( "BOARD INITIALIZATION:\n" );

	dsccb.boardtype = DSC_HERCEBX;

    // The Hercules II has a fixed base address.
    dsccb.base_address = 0x240;

	dsccb.int_level = 5;

	if ( dscInitBoard ( dsccb.boardtype, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	//III. AUTOCAL SETTINGS INITIALIZATION
	//
	//		Initialize the structure containing the DA autocal settings. The
	//		structure element, dscautocal.darange, contains a code ranging from
	//		0-4095at scales to a voltage based on the following formula:
	//
	//
	//		STEPS TO FOLLOW:
	//
	//		1. set the DA range code
	//		2. set the programmable voltage
	//=========================================================================

	printf ( "\nAUTOCAL SETTINGS INITIALIZATION:\n" );

	memset(&dscdacalparams, 0, sizeof(DSCDACALPARAMS));

	dscdacalparams.darange = 10;
	dscdacalparams.fixed = TRUE;

    // The Hercules II boots up the D/A in unipolar mode; user may change that here..

    printf ("Enter the D/A polarity (0 for BIPOLAR, 1 for UNIPOLAR; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscdacalparams.polarity = 1;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	dscdacalparams.polarity = (BYTE) intBuff;
	}

	//=========================================================================
	// IV. AUTO-CALIBRATION
	//
	//	   Perform the actual DA auto-calibration of the board.
	//
	//	   STEPS TO FOLLOW:
	//
	//	   1. perform auto-calibration
	//=========================================================================

	printf ( "\nAUTO-CALIBRATION:\n" );
	printf ( "Starting DA auto-calibration...\n" );

	if ( dscDAAutoCal ( dscb, &dscdacalparams ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscDAAutoCal error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// V. CALIBRATION VERIFICATION
	//
	//	  To verify that the auto-calibration was successfully completed, you
	//	  should run dscDACalVerify() to ensure that the gain and offset errors
	//	  meet a specified tolerance (2 in this example).
	//
	//	  STEPS TO FOLLOW:
	//
	//	  1. verify that the calibration was successful and output results
	//=========================================================================

	printf ( "\nCALIBRATION VERIFICATION:\n" );

	if ( dscDACalVerify ( dscb, &dscdacalparams ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscDACalVerify error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	printf ( "Offset Error: %9.3f, Gain Error: %9.3f\n", dscdacalparams.offset, dscdacalparams.gain ); 

	if ( fabs( dscdacalparams.offset ) > OFFSET_LSB_TOLERANCE || 
		 fabs( dscdacalparams.gain ) > GAIN_LSB_TOLERANCE )
		printf ( "Value for offset or gain exceeded specified tolerance\n" );
	else printf ( "Values for offset and gain met specified tolerance\n" );

	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();


	printf ( "\nDSCDAAutoCal completed. \n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
